import hashlib
import json
from datetime import datetime, timedelta
import re
from typing import Optional

import requests
import xmltodict

from iikoserver_api.schemas.account import Account
from iikoserver_api.schemas.cashshift import Cashshift
from iikoserver_api.schemas.conception import Conception
from iikoserver_api.schemas.corporation import CorporationStore
from iikoserver_api.schemas.employee import Employee
from iikoserver_api.schemas.incoming_invoice import IncomingInvoice
from iikoserver_api.schemas.nomenclature_group import NomenclatureGroup
from iikoserver_api.schemas.outgoing_invoice import OutgoingInvoice
from iikoserver_api.schemas.product import Product
from iikoserver_api.schemas.write_off import WriteOff
from iikoserver_api.services.implementation_act_builder import ImplementationActBuilder
from iikoserver_api.services.incoming_service_builder import IncomingServiceBuilder
from iikoserver_api.services.inventory_document_builder import InventoryDocumentBuilder
from iikoserver_api.services.olap_cash_shift_builder import OlapCashShiftBuilder, OlapCashShiftBuilderV2
from iikoserver_api.services.outgoing_service_builder import OutgoingServiceBuilder
from iikoserver_api.services.returning_invoice_builder import ReturningInvoiceBuilder


class IikoServerApi:
    # _default_headers = {'Content-Type': 'application/xml'}

    def __init__(self, url, login, password):
        self.url = url
        self.login = login
        self.password = hashlib.sha1(password.encode()).hexdigest()
        self.token = None
        self.connected = False

    def send(self,
             action: str,
             method: str = 'GET',
             params: dict = None,
             headers: dict = None,
             data: any = None,
             json=None):
        # if headers is None:
        #     headers = self._default_headers
        url = f'{self.url}{action}/'
        response = requests.request(method=method, url=url, params=params, headers=headers, data=data, json=json)

        if response.ok:
            if action.startswith('v2'):
                return response.json()
            return response.content
        else:
            error_info = {
                "response_status": response.status_code,
                "response_info": response.content.decode()
            }
            raise ConnectionError(error_info)

    def auth(self):
        params = {
            'login': self.login,
            'pass': self.password,
        }
        response = self.send(action='auth', params=params)
        data = response.decode()
        self.token = data
        self.connected = True

    def logout(self):
        params = {
            'key': self.token
        }
        data = self.send(action='logout', params=params)
        self.token = None
        self.connected = False
        return data

    def export_outgoing_invoice(self, from_datetime: datetime,
                                to_datetime: datetime) -> Optional[list[OutgoingInvoice]]:
        params = {
            'key': self.token,
            'from': from_datetime.strftime('%Y-%m-%d'),
            'to': to_datetime.strftime('%Y-%m-%d'),
        }
        data = self.send(action='documents/export/outgoingInvoice', params=params)
        xml_dict_data = xmltodict.parse(data.decode())
        documents = xml_dict_data['outgoingInvoiceDtoes']
        if documents:
            documents = [documents['document']] if type(documents['document']) is dict else documents['document']
        else:
            return
        result = []
        for document in documents:
            document_items = document['items']
            if not document_items:
                continue
            document_items = document_items.get('item')
            if document_items:
                document_items = [document['items']['item']] \
                    if type(document['items']['item']) is dict else document['items']['item']
            else:
                continue
            dict_data = {'items': []}
            for key, value in document.items():
                if key == 'items':
                    continue
                key = re.sub(r'(?<!^)(?=[A-Z])', '_', key).lower()
                dict_data[key] = value

            for item in document_items:
                new_item = {}
                for key, value in item.items():
                    key = re.sub(r'(?<!^)(?=[A-Z])', '_', key).lower()
                    new_item[key] = value

                dict_data['items'].append(new_item)

            result.append(OutgoingInvoice(**dict_data))

        return result

    def export_incoming_invoice(self, from_datetime: datetime,
                                to_datetime: datetime) -> Optional[list[IncomingInvoice]]:
        params = {
            'key': self.token,
            'from': from_datetime.strftime('%Y-%m-%d'),
            'to': to_datetime.strftime('%Y-%m-%d'),
        }
        data = self.send(action='documents/export/incomingInvoice', params=params)
        xml_dict_data = xmltodict.parse(data.decode())
        documents = xml_dict_data['incomingInvoiceDtoes']
        if documents:
            documents = [documents['document']] if type(documents['document']) is dict else documents['document']
        else:
            return
        result = []
        for document in documents:
            document_items = document['items']
            if not document_items:
                continue
            document_items = document_items.get('item')
            if document_items:
                document_items = [document['items']['item']] \
                    if type(document['items']['item']) is dict else document['items']['item']
            else:
                continue
            dict_data = {'items': []}
            for key, value in document.items():
                if key == 'items':
                    continue
                key = re.sub(r'(?<!^)(?=[A-Z])', '_', key).lower()
                dict_data[key] = value

            for item in document_items:
                new_item = {}
                for key, value in item.items():
                    key = re.sub(r'(?<!^)(?=[A-Z])', '_', key).lower()
                    new_item[key] = value

                dict_data['items'].append(new_item)

            result.append(IncomingInvoice(**dict_data))

        return result or []

    def get_employee(self, pk: str) -> Optional[Employee]:
        params = {
            'key': self.token,
            # 'from': from_datetime.strftime('%Y-%m-%d'),
            # 'to': to_datetime.strftime('%Y-%m-%d'),
        }
        data = self.send(action=f'employees/byId/{pk}', params=params)
        xml_dict_data = xmltodict.parse(data.decode())
        employee = xml_dict_data['employee']
        dict_data = {}
        for key, value in employee.items():
            key = re.sub(r'(?<!^)(?=[A-Z])', '_', key).lower()
            if value in ['true', 'false']:
                value = True if value == 'true' else False
            dict_data[key] = value

        return Employee(**dict_data)
        # documents = xml_dict_data['incomingInvoiceDtoes']

    def export_nomenclature_group(self):
        params = {
            'key': self.token,
        }
        data = self.send(action='v2/entities/products/group/list', params=params)
        result = []
        for el in data:
            result.append(NomenclatureGroup(**el))
        return result

    def export_products(self):
        params = {
            'key': self.token,
        }
        data = self.send(action='v2/entities/products/list', params=params)
        result = []
        for el in data:
            result.append(Product(**el))
        return result

    def export_cashshifts(self, from_datetime: datetime, to_datetime: datetime, status: str = 'ANY'):
        params = {
            'key': self.token,
            'openDateFrom': from_datetime.strftime('%Y-%m-%d'),
            'openDateTo': to_datetime.strftime('%Y-%m-%d'),
            'status': status,
        }
        data = self.send(action='v2/cashshifts/list', params=params)
        result = []
        for el in data:
            result.append(Cashshift(**el))
        return result

    def export_cashshift_payments(self, pk: str):
        params = {
            'key': self.token,
            'hideAccepted': False
        }
        data = self.send(action=f'v2/cashshifts/payments/list/{pk}', params=params)
        return data

    def export_stores(self) -> list[CorporationStore]:
        params = {
            'key': self.token,
        }
        data = self.send(action='corporation/stores', params=params)
        xml_dict_data = xmltodict.parse(data.decode())
        stores = xml_dict_data['corporateItemDtoes']['corporateItemDto']
        results = []
        for store in stores:
            dict_data = {}
            for key, value in store.items():
                key = re.sub(r'(?<!^)(?=[A-Z])', '_', key).lower()
                if value in ['true', 'false']:
                    value = True if value == 'true' else False
                dict_data[key] = value
            results.append(CorporationStore(**dict_data))

        return results

    def get_terminals(self):
        params = {
            'key': self.token,
        }
        data = self.send('corporation/departments', params=params)
        xml_dict_data = xmltodict.parse(data.decode())
        return xml_dict_data

    def export_pal(self):
        params = {
            'key': self.token,
            'reportType': 'TRANSACTIONS'
        }
        end_date = datetime.now()
        start_date = end_date - timedelta(days=3)
        data = {
            "reportType": "TRANSACTIONS",
            "buildSummary": "false",
            "groupByRowFields": [],
            "groupByColFields": [
                "Account.Name",
                "Account.Id",
                "Account.StoreOrAccount",
            ],
            "filters": {
                "DateTime.DateTyped": {
                    "filterType": "DateRange",
                    "periodType": "CUSTOM",
                    "from": start_date.strftime('%Y-%m-%d'),
                    "to": end_date.strftime('%Y-%m-%d'),
                    "includeLow": True,
                    "includeHigh": True
                }
            }
        }
        data = self.send('v2/reports/olap', method='POST', json=data, params=params)
        return data

    def export_write_offs(self, from_datetime: datetime, to_datetime: datetime,):
        params = {
            'key': self.token,
            'dateFrom': from_datetime.strftime('%Y-%m-%d'),
            'dateTo': to_datetime.strftime('%Y-%m-%d'),
        }
        data = self.send(action=f'v2/documents/writeoff', params=params)
        result = []
        for el in data.get('response', []):
            result.append(WriteOff(**el))
        return result

    def export_accounts(self):
        params = {
            'key': self.token,
            'reportType': 'TRANSACTIONS'
        }
        end_date = datetime.now() + timedelta(days=1)
        start_date = end_date - timedelta(days=90)
        json = {
            "reportType": "TRANSACTIONS",
            "buildSummary": "false",
            "groupByRowFields": [],
            "groupByColFields": [
                "Account.Name",
                "Account.Id",
                "Account.StoreOrAccount",
            ],
            "filters": {
                "DateTime.DateTyped": {
                    "filterType": "DateRange",
                    "periodType": "CUSTOM",
                    "from": start_date.strftime('%Y-%m-%d'),
                    "to": end_date.strftime('%Y-%m-%d'),
                    "includeLow": True,
                    "includeHigh": True
                },
                "Account.StoreOrAccount": {
                    "filterType": "IncludeValues",
                    "values": ["ACCOUNT"]
                }
            }
        }
        data = self.send('v2/reports/olap', method='POST', json=json, params=params)
        result = []
        for item in data.get('data', []):
            result.append(Account(**item))
        return result

    def export_suppliers(self) -> list[Employee]:
        params = {
            'key': self.token,
        }
        data = self.send(action='suppliers', params=params)
        xml_dict_data = xmltodict.parse(data.decode())
        employees = xml_dict_data['employees']['employee']
        result = []
        for employee in employees:
            dict_data = {}
            for key, value in employee.items():
                key = re.sub(r'(?<!^)(?=[A-Z])', '_', key).lower()
                if value in ['true', 'false']:
                    value = True if value == 'true' else False
                dict_data[key] = value
            result.append(Employee(**dict_data))

        return result

    def export_employees(self) -> list[Employee]:
        params = {
            'key': self.token,
        }
        data = self.send(action='employees', params=params)
        xml_dict_data = xmltodict.parse(data.decode())
        employees = xml_dict_data['employees']['employee']
        result = []
        for employee in employees:
            dict_data = {}
            for key, value in employee.items():
                key = re.sub(r'(?<!^)(?=[A-Z])', '_', key).lower()
                if value in ['true', 'false']:
                    value = True if value == 'true' else False
                dict_data[key] = value
            result.append(Employee(**dict_data))

        return result

    def export_inventory(self, from_date: datetime = datetime.now() - timedelta(days=30), to_date: datetime = datetime.now()):
        params = {
            'key': self.token,
            'documentTypes': ['INCOMING_INVENTORY'],
            'dateFrom': from_date.strftime('%d.%m.%Y'),
            'dateTo': to_date.strftime('%d.%m.%Y'),
        }
        data = self.send(action='reports/storeOperations', params=params)
        xml_dict_data = xmltodict.parse(data.decode())
        items = xml_dict_data.get('storeReportItemDtoes') or {}
        items = items.get('storeReportItemDto', {})
        return InventoryDocumentBuilder(items).process()

    def export_incoming_service(self, from_date: datetime = datetime.now() - timedelta(days=3), to_date: datetime = datetime.now()):
        params = {
            'key': self.token,
            'reportType': 'TRANSACTIONS'
        }
        json = {
            "reportType": "TRANSACTIONS",
            "buildSummary": "false",
            "groupByRowFields": [],
            "groupByColFields": [
                "Document",
                "Account.Id",
                "Product.Id",
                "DateTime.Typed",
                "Counteragent.Id",
                "Conception.Code",
                "Session.GroupId",
            ],
            "filters": {
                "DateTime.DateTyped": {
                    "filterType": "DateRange",
                    "periodType": "CUSTOM",
                    "from": from_date.strftime('%Y-%m-%d'),
                    "to": to_date.strftime('%Y-%m-%d'),
                    "includeLow": True,
                    "includeHigh": True
                },
                "TransactionType": {
                    "filterType": "IncludeValues",
                    "values": ["INCOMING_SERVICE"]
                }
            },
            "aggregateFields": [
                "Sum.Incoming",
                "Sum.Outgoing",
                "Sum.ResignedSum",
                "Amount.Out",
                "Amount.In",
                "Amount"
            ]
        }
        data = self.send('v2/reports/olap', method='POST', json=json, params=params)
        items = data.get('data', [])
        return IncomingServiceBuilder(items).process()

    def export_outgoing_service(self, from_date: datetime = datetime.now() - timedelta(days=3), to_date: datetime = datetime.now() + timedelta(days=1)):
        params = {
            'key': self.token,
            'reportType': 'TRANSACTIONS'
        }
        json = {
            "reportType": "TRANSACTIONS",
            "buildSummary": "false",
            "groupByRowFields": [],
            "groupByColFields": [
                "Document",
                "Account.Id",
                "Product.Id",
                "DateTime.Typed",
                "Counteragent.Id",
                "Account.CounteragentType",
                "Conception.Code",
                "Session.GroupId",
            ],
            "filters": {
                "DateTime.DateTyped": {
                    "filterType": "DateRange",
                    "periodType": "CUSTOM",
                    "from": from_date.strftime('%Y-%m-%d'),
                    "to": to_date.strftime('%Y-%m-%d'),
                    "includeLow": True,
                    "includeHigh": True
                },
                "TransactionType": {
                    "filterType": "IncludeValues",
                    "values": ["OUTGOING_SERVICE"]
                }
            },
            "aggregateFields": [
                "Sum.Incoming",
                "Sum.Outgoing",
                "Sum.ResignedSum",
                "Amount.Out",
                "Amount.In",
                "Amount"
            ]
        }
        data = self.send('v2/reports/olap', method='POST', json=json, params=params)
        items = data.get('data', [])
        return OutgoingServiceBuilder(items).process()

    def export_columns(self):
        params = {
            'key': self.token,
            'reportType': 'SALES'
        }
        data = self.send('v2/reports/olap/columns', method='GET', params=params)
        return data

    def export_implementation_act(self, from_date: datetime = datetime.now() - timedelta(days=2), to_date: datetime = datetime.now()):
        params = {
            'key': self.token,
            'reportType': 'TRANSACTIONS'
        }
        json = {
            "reportType": "TRANSACTIONS",
            "buildSummary": "false",
            "groupByRowFields": ["TransactionType", "Account.Id", "Account.StoreOrAccount", "Account.Name"],
            "groupByColFields": [
                "Document",
                "Product.Id",
                "Session.GroupId",
                "DateTime.Typed",
                "Conception.Code",
            ],
            "filters": {
                "DateTime.DateTyped": {
                    "filterType": "DateRange",
                    "periodType": "CUSTOM",
                    "from": from_date.strftime('%Y-%m-%d'),
                    "to": to_date.strftime('%Y-%m-%d'),
                    "includeLow": True,
                    "includeHigh": True
                },
                "TransactionType": {
                    "filterType": "IncludeValues",
                    "values": ["SALES_REVENUE", "SESSION_WRITEOFF"]
                }
            },
            "aggregateFields": [
                "Sum.Incoming",
                "Sum.Outgoing",
                "Sum.ResignedSum",
                "Amount.Out",
                "Amount.In",
                "Amount"
            ]
        }
        data = self.send('v2/reports/olap', method='POST', json=json, params=params)
        items = data.get('data', [])
        result = ImplementationActBuilder(items).process()
        return result

    def export_olap_cash(self, from_date: datetime = datetime.now() - timedelta(days=2), to_date: datetime = datetime.now()):
        params = {
            'key': self.token,
            'reportType': 'TRANSACTIONS'
        }
        json = {
            "reportType": "TRANSACTIONS",
            "buildSummary": "false",
            "groupByRowFields": [],
            "groupByColFields": [
                "Document",
                "Session.CashRegister",
                "DateTime.Typed",
            ],
            "filters": {
                "DateTime.DateTyped": {
                    "filterType": "DateRange",
                    "periodType": "CUSTOM",
                    "from": from_date.strftime('%Y-%m-%d'),
                    "to": to_date.strftime('%Y-%m-%d'),
                    "includeLow": True,
                    "includeHigh": True
                },
                "TransactionType": {
                    "filterType": "IncludeValues",
                    "values": ["CASH"]
                }
            },
            "aggregateFields": [
                "Sum.Incoming",
                "Sum.Outgoing",
                "Sum.ResignedSum",
            ]
        }
        data = self.send('v2/reports/olap', method='POST', json=json, params=params)
        items = data.get('data', [])
        return OlapCashShiftBuilder(items).process()

    def export_olap_cash_2(self, from_date: datetime = datetime.now() - timedelta(days=2), to_date: datetime = datetime.now()):
        params = {
            'key': self.token,
            'reportType': 'SALES'
        }
        json = {
            "reportType": "SALES",
            "buildSummary": "false",
            "groupByRowFields": [],
            "groupByColFields": [
                "CashRegisterName",
                "CashRegisterName.Number",
                "CashRegisterName.CashRegisterSerialNumber",
                "OpenDate.Typed",
                "PayTypes",
                "SessionNum",
            ],
            "filters": {
                "OpenDate.Typed": {
                    "filterType": "DateRange",
                    "periodType": "CUSTOM",
                    "from": from_date.strftime('%Y-%m-%d'),
                    "to": to_date.strftime('%Y-%m-%d'),
                    "includeLow": True,
                    "includeHigh": True
                },
                "PayTypes": {
                    "filterType": "IncludeValues",
                    "values": ["Наличные"]
                }
            },
            "aggregateFields": [
                "DishDiscountSumInt",
            ]
        }
        data = self.send('v2/reports/olap', method='POST', json=json, params=params)
        items = data.get('data', [])
        return OlapCashShiftBuilderV2(items).process()

    def export_conceptions_deprecated(self, from_date: datetime = datetime.now() - timedelta(days=365), to_date: datetime = datetime.now()):
        params = {
            'key': self.token,
            'reportType': 'TRANSACTIONS'
        }
        json = {
            "reportType": "TRANSACTIONS",
            "buildSummary": "false",
            "groupByRowFields": [],
            "groupByColFields": [
                "Conception.Code",
                "Conception",
            ],
            "filters": {
                "DateTime.DateTyped": {
                    "filterType": "DateRange",
                    "periodType": "CUSTOM",
                    "from": from_date.strftime('%Y-%m-%d'),
                    "to": to_date.strftime('%Y-%m-%d'),
                    "includeLow": True,
                    "includeHigh": True
                },
                "TransactionType": {
                    "filterType": "IncludeValues",
                    "values": ["CASH"]
                }
            },
            "aggregateFields": []
        }
        data = self.send('v2/reports/olap', method='POST', json=json, params=params)
        items = data.get('data', [])
        result = []
        for item in items:
            result.append(Conception(
                id=item.get('Conception.Code'),
                name=item.get('Conception')
            ))
        return result

    def export_returning_invoice(self, from_date: datetime = datetime.now() - timedelta(days=2), to_date: datetime = datetime.now()):
        params = {
            'key': self.token,
            'reportType': 'TRANSACTIONS'
        }
        json = {
            "reportType": "TRANSACTIONS",
            "buildSummary": "false",
            "groupByRowFields": [],
            "groupByColFields": [
                "Document",
                "DateTime.Typed",
                "Conception.Code",
                "Counteragent.Id",
                "TransactionType",
                "Account.Id",
                "Account.StoreOrAccount",
                "Account.Name",
                "Product.Id",
                "Product.Name",
                "Session.GroupId",
            ],
            "filters": {
                "DateTime.DateTyped": {
                    "filterType": "DateRange",
                    "periodType": "CUSTOM",
                    "from": from_date.strftime('%Y-%m-%d'),
                    "to": to_date.strftime('%Y-%m-%d'),
                    "includeLow": True,
                    "includeHigh": True
                },
                "TransactionType": {
                    "filterType": "IncludeValues",
                    "values": ["RETURNED_INVOICE_COST_AFFECTED"]
                }
            },
            "aggregateFields": [
                "Sum.Incoming",
                "Sum.Outgoing",
                "Sum.ResignedSum",
                "Amount.Out",
                "Amount.In",
                "Amount"
            ]
        }
        data = self.send('v2/reports/olap', method='POST', json=json, params=params)
        items = data.get('data', [])
        return ReturningInvoiceBuilder(items).process()

    def export_conceptions(self):
        params = {
            'key': self.token,
            'rootType': 'Conception'
        }
        data = self.send('v2/entities/list', method='GET', params=params)
        result = []
        for item in data:
            result.append(Conception(**item))
        return result

    def __enter__(self):
        self.auth()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.logout()



import hashlib
from datetime import datetime, timedelta

import requests


class RequestManager:
    _default_headers = {'Content-Type': 'application/xml'}

    def __init__(self, url: str = None, ):
        self.url = url
        self.token = None

    def send(self, action: str, method: str = 'GET', params: dict = None, headers: dict = None, data: any = None,
             json=None):
        if headers is None:
            headers = self._default_headers
        url = f'{self.url}{action}/'
        response = requests.request(method=method, url=url, params=params, headers=headers, data=data, json=json)

        if response.ok:
            return response.content
        else:
            error_info = {
                "response_status": response.status_code,
                "response_info": response.content.decode()
            }
            # if action not in ['logout', 'auth']:
            #     self.clear_token()
            raise ConnectionError(error_info)

    def test_send(self):
        params = {
            'key': self.token,
            'from': (datetime.now() - timedelta(days=1000)).strftime('%Y-%m-%d'),
            'to': datetime.now().strftime('%Y-%m-%d'),
        }
        data = self.send(action='documents/export/incomingInvoice', params=params)

    def auth(self, login, password):
        password = hashlib.sha1(password.encode()).hexdigest()
        params = {
            'login': login,
            'pass': password,
        }
        response = self.send(action='auth', params=params)
        data = response.decode()
        self.token = data

    def clear_token(self):
        params = {
            'key': self.token
        }
        data = self.send(action='logout', params=params)
        self.token = None






from datetime import datetime
from typing import Optional

from pydantic import BaseModel


class OutgoingServiceItem(BaseModel):
    num: Optional[str] = None
    product: Optional[str] = None
    account: Optional[str] = None
    amount: Optional[float] = None
    sum: Optional[float] = None


class OutgoingService(BaseModel):
    id: str
    date: Optional[datetime] = None
    num: Optional[str] = None
    account: Optional[str] = None
    counterparty: Optional[str] = None
    counterparty_type: Optional[str] = None
    conception_code: Optional[str] = None
    items: list[OutgoingServiceItem] = []




from typing import Optional

from pydantic import BaseModel


class IncomingInvoiceItem(BaseModel):
    actual_amount: Optional[float] = None
    store: Optional[str] = None
    code: Optional[str] = None
    price: Optional[float] = None
    sum: Optional[float] = None
    vat_percent: Optional[float] = None
    vat_sum: Optional[float] = None
    discount_sum: Optional[float] = None
    amount_unit: Optional[str] = None
    num: Optional[int] = None
    product: Optional[str] = None
    product_article: Optional[str] = None
    amount: Optional[float] = None


class IncomingInvoice(BaseModel):
    id: str
    items: list[IncomingInvoiceItem] = []
    incoming_document_number: Optional[str] = None
    incoming_date: Optional[str] = None
    use_default_document_time: Optional[bool] = None
    due_date: Optional[str] = None
    supplier: Optional[str] = None
    default_store: Optional[str] = None
    document_number: Optional[str] = None
    invoice: Optional[str] = None
    comment: Optional[str] = None
    status: Optional[str] = None
    conception_code: Optional[str] = None
    conception: Optional[str] = None






from enum import StrEnum, auto
from typing import Optional

from pydantic import BaseModel


class InventoryItemEnum(StrEnum):
    INCOMING = auto()
    OUTGOING = auto()


class InventoryItem(BaseModel):
    sum: Optional[float] = None
    type: Optional[InventoryItemEnum] = None
    account: Optional[str] = None


class Inventory(BaseModel):
    id: str
    store: Optional[str] = None
    num: Optional[str] = None
    date: Optional[str] = None
    incoming_sum: Optional[float] = None
    outgoing_sum: Optional[float] = None
    sum: Optional[float] = None
    items: list[InventoryItem] = []







from typing import Optional

from pydantic import BaseModel, ConfigDict
from pydantic.alias_generators import to_camel


class Product(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel, use_enum_values=True)
    id: Optional[str]
    deleted: Optional[bool] = None
    name: Optional[str] = None
    description: Optional[str] = None
    num: Optional[str] = None
    code: Optional[str] = None
    parent: Optional[str] = None
    modifiers: Optional[list] = None
    tax_category: Optional[str] = None
    category: Optional[str] = None
    accounting_category: Optional[str] = None
    front_image_id: Optional[str] = None
    position: Optional[int] = None
    main_unit: Optional[str] = None
    excluded_sections: Optional[set] = None
    default_sale_price: Optional[float] = None
    place_type: Optional[str] = None
    default_include_in_menu: Optional[bool] = None
    type: Optional[str] = None
    unit_weight: Optional[float] = None
    unit_capacity: Optional[float] = None
    not_in_store_movement: Optional[bool] = None





from typing import Optional

from pydantic import BaseModel, ConfigDict
from pydantic.alias_generators import to_camel


class WriteOffItem(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel, use_enum_values=True)
    num: int
    product_id: Optional[str] = None
    product_size_id: Optional[str] = None
    amount_factor: Optional[float] = None
    amount: Optional[float] = None
    measure_unit_id: Optional[str] = None
    container_id: Optional[str] = None
    cost: Optional[float] = None


class WriteOff(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel, use_enum_values=True)
    id: Optional[str]
    date_incoming: Optional[str] = None
    document_number: Optional[str] = None
    status: Optional[str] = None
    conception_id: Optional[str] = None
    comment: Optional[str] = None
    store_id: Optional[str] = None
    account_id: Optional[str] = None
    external_outgoing_invoice_id: Optional[str] = None
    external_production_document_id: Optional[str] = None
    items: list[WriteOffItem] = []




from typing import Optional

from pydantic import BaseModel, ConfigDict
from pydantic.alias_generators import to_pascal


def account_alias_generator(string: str) -> str:
    return 'Account.' + to_pascal(string)


class Account(BaseModel):
    model_config = ConfigDict(alias_generator=account_alias_generator, use_enum_values=True)
    id: str
    name: Optional[str] = None





import json
from collections import defaultdict
from datetime import datetime

from iikoserver_api.schemas.implementation_act import ImplementationActItem, ImplementationActItemTypeEnum, \
    ImplementationAct


class ImplementationActBuilder:
    def __init__(self, items):
        self.items = items

    def process(self):
        document = defaultdict(dict)
        for item in self.items:
            document_id = f'{item['Document']}_{item['DateTime.Typed']}_5_{item.get('Session.GroupId', '0')}'
            account = item.get('Account.Id')
            product = item.get('Product.Id')
            transaction_type = item.get('TransactionType')
            if not document[document_id].get(account):
                document[document_id][account] = defaultdict(dict)
            if not document[document_id][account].get(product):
                document[document_id][account][product] = defaultdict(list)
            document[document_id][account][product][transaction_type].append(item)

        result = []
        for document_id, accounts in document.items():
            main_document = None
            store = None
            account = None
            conception = None
            document = {
                'id': document_id,
            }
            document_items = []
            for account_id, products in accounts.items():
                for product_id, transactions in products.items():
                    for transaction_type, items in transactions.items():
                        for item in items:
                            if not conception:
                                conception = item.get('Conception.Code')
                            if transaction_type == 'SALES_REVENUE':
                                if not item.get('Sum.Outgoing'):
                                    continue
                                if not account:
                                    account = account_id
                                data = {
                                    'type': ImplementationActItemTypeEnum.INCOMING,
                                    'sum': item.get('Sum.Outgoing'),
                                    'amount': abs(item.get('Amount')),
                                    'account': account_id,
                                }
                            else:
                                if not item.get('Account.StoreOrAccount') == 'STORE' or not item.get('Sum.Outgoing'):
                                    continue
                                if not store:
                                    store = account_id
                                data = {
                                    'type': ImplementationActItemTypeEnum.OUTGOING,
                                    'sum': item.get('Sum.Outgoing'),
                                    'amount': item.get('Amount.Out'),
                                    'store': account_id
                                }
                            if not main_document:
                                main_document = item

                            document_items.append(ImplementationActItem(
                                product=product_id,
                                **data
                            ))

            if not main_document:
                continue

            document['num'] = main_document.get('Document')
            document['date'] = datetime.fromisoformat(main_document.get('DateTime.Typed'))
            document['store'] = store
            document['account'] = account
            document['items'] = document_items
            document['conception_code'] = conception

            result.append(ImplementationAct(
                **document
            ))

        return result




from typing import Optional

from pydantic import BaseModel


class OutgoingInvoiceItem(BaseModel):
    product_id: Optional[str] = None
    product_article: Optional[str] = None
    store_id: Optional[str] = None
    store_code: Optional[str] = None
    price: Optional[float] = None
    amount: Optional[float] = None
    sum: Optional[float] = None
    discount_sum: Optional[float] = None
    vat_percent: Optional[float] = None
    vat_sum: Optional[float] = None


class OutgoingInvoice(BaseModel):
    id: str
    document_number: Optional[str] = None
    date_incoming: Optional[str] = None
    use_default_document_time: Optional[bool] = None
    status: Optional[str] = None
    account_to_code: Optional[str] = None
    revenue_account_code: Optional[str] = None
    default_store_id: Optional[str] = None
    default_store_code: Optional[str] = None
    counteragent_id: Optional[str] = None
    counteragent_code: Optional[str] = None
    comment: Optional[str] = None
    conception_code: Optional[str] = None
    conception_id: Optional[str] = None
    items: list[OutgoingInvoiceItem] = []




from datetime import datetime
from typing import Optional

from pydantic import BaseModel, ConfigDict
from pydantic.alias_generators import to_camel


class Cashshift(BaseModel):
    model_config = ConfigDict(alias_generator=to_camel, use_enum_values=True)
    id: Optional[str]
    session_number: Optional[int] = None
    fiscal_number: Optional[int] = None
    cash_reg_number: Optional[int] = None
    cash_reg_serial: Optional[str] = None
    open_date: Optional[str] = None
    close_date: Optional[str] = None
    accept_date: Optional[str] = None
    manager_id: Optional[str] = None
    responsible_user: Optional[str] = None
    session_start_cash: Optional[float] = None
    pay_orders: Optional[float] = None
    sum_writeoff_orders: Optional[float] = None
    sales_cash: Optional[float] = None
    sales_credit: Optional[float] = None
    sales_card: Optional[float] = None
    pay_in: Optional[float] = None
    pay_out: Optional[float] = None
    pay_income: Optional[float] = None
    cash_remain: Optional[float] = None
    cash_diff: Optional[float] = None
    session_status: Optional[str] = None
    conception: Optional[str] = None
    point_of_sale: Optional[str] = None


class OlapCashShift(BaseModel):
    id: str
    session_number: Optional[int] = None
    cash_shift_name: Optional[str] = None
    close_date: Optional[datetime] = None
    amount: Optional[float] = None


class OlapCashShiftV2(BaseModel):
    id: str
    session_number: Optional[str] = None
    cash_shift_name: Optional[str] = None
    cash_shift_number: Optional[str] = None
    cash_shift_serial: Optional[str] = None
    cash_shift_id: Optional[str] = None
    close_date: Optional[datetime] = None
    amount: Optional[float] = None



from typing import Optional

from pydantic import BaseModel


class Employee(BaseModel):
    id: str
    code: Optional[str] = None
    name: Optional[str] = None
    login: Optional[str] = None
    phone: Optional[str] = None
    cell_phone: Optional[str] = None
    first_name: Optional[str] = None
    middle_name: Optional[str] = None
    last_name: Optional[str] = None
    note: Optional[str] = None
    card_number: Optional[str] = None
    taxpayer_id_number: Optional[str] = None
    snils: Optional[str] = None
    preferred_department_code: Optional[str] = None
    deleted: Optional[bool] = None
    supplier: Optional[bool] = None
    employee: Optional[bool] = None
    client: Optional[bool] = None
    represents_store: Optional[bool] = None




from datetime import datetime
from enum import StrEnum, auto
from typing import Optional

from pydantic import BaseModel


class ImplementationActItemTypeEnum(StrEnum):
    INCOMING = auto()
    OUTGOING = auto()


class ImplementationActItem(BaseModel):
    type: Optional[ImplementationActItemTypeEnum] = None
    sum: Optional[float] = None
    amount: Optional[float] = None
    store: Optional[str] = None
    account: Optional[str] = None
    product: Optional[str] = None


class ImplementationAct(BaseModel):
    id: str
    num: Optional[str] = None
    date: Optional[datetime] = None
    store: Optional[str] = None
    account: Optional[str] = None
    conception_code: Optional[str] = None
    items: list[ImplementationActItem] = []




from setuptools import setup, find_packages

setup(
    name="iikoserver_api",
    version="0.0.51",
    packages=find_packages(),
    install_requires=['pydantic>=2.0.0', 'requests>=2.22.0', 'xmltodict>=0.12.0'],
    author="Eugene",
    author_email="zenjagems@gmail.com",
    description="iikoserver api connector",
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url="https://github.com/sizlik/iikoserver_api",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.12',
)




from datetime import datetime
from typing import Optional

from pydantic import BaseModel


class ReturningInvoiceItem(BaseModel):
    num: str
    amount: Optional[float] = None
    product: Optional[str] = None
    sum: Optional[float] = None
    store: Optional[str] = None
    account: Optional[str] = None


class ReturningInvoice(BaseModel):
    id: str
    number: Optional[str] = None
    date: Optional[datetime] = None
    account: Optional[str] = None
    counterparty: Optional[str] = None
    conception_code: Optional[str] = None
    items: list[ReturningInvoiceItem] = []



from datetime import datetime

from iikoserver_api.schemas.cashshift import OlapCashShift, OlapCashShiftV2


class OlapCashShiftBuilder:
    def __init__(self, items: list[dict]):
        self.items = items

    def process(self):
        result = []
        for item in self.items:
            id = f'{item.get('Document')}_{item.get('DateTime.Typed')}_olap'
            result.append(
                OlapCashShift(
                    id=id,
                    session_number=item.get('Document'),
                    cash_shift_name=item.get('Session.CashRegister'),
                    close_date=datetime.fromisoformat(item.get('DateTime.Typed')),
                    amount=item.get('Sum.Incoming')
                )
            )

        return result


class OlapCashShiftBuilderV2:
    def __init__(self, items: list[dict]):
        self.items = items

    def process(self):
        result = []
        for item in self.items:
            cash_shift_id = f'{item.get('CashRegisterName')}_{item.get('CashRegisterName.Number')}_{item.get('CashRegisterName.CashRegisterSerialNumber')}'
            id = f'olap_{item.get('SessionNum')}_{item.get('OpenDate.Typed')}_{cash_shift_id}'
            result.append(
                OlapCashShiftV2(
                    id=id,
                    session_number=str(item.get('SessionNum')),
                    cash_shift_name=str(item.get('CashRegisterName')),
                    cash_shift_number=str(item.get('CashRegisterName.Number')),
                    cash_shift_serial=str(item.get('CashRegisterName.CashRegisterSerialNumber')),
                    cash_shift_id=cash_shift_id,
                    close_date=datetime.fromisoformat(item.get('OpenDate.Typed')),
                    amount=item.get('DishDiscountSumInt')
                )
            )

        return result



from typing import Optional

from pydantic import BaseModel


class CorporationStore(BaseModel):
    id: Optional[str] = None
    parent_id: Optional[str] = None
    code: Optional[str] = None
    name: Optional[str] = None
    type: Optional[str] = None




import json
from collections import defaultdict
from datetime import datetime

from iikoserver_api.schemas.incoming_service import IncomingServiceItem, IncomingService
from iikoserver_api.schemas.returning_invoice import ReturningInvoiceItem, ReturningInvoice


class ReturningInvoiceBuilder:
    def __init__(self, items):
        self.items = items

    def process(self):
        documents = defaultdict(dict)
        for item in self.items:
            document_id = f'{item.get('Document')}_{item.get('DateTime.Typed')}_8_{item.get('Session.GroupId', '0')}'
            if not documents[document_id].get(item['Product.Id']):
                documents[document_id][item['Product.Id']] = [item]
            else:
                documents[document_id][item['Product.Id']].append(item)

        result = []
        for document, products in documents.items():
            document_items = []
            main_item = None
            for index, (product, items) in enumerate(products.items()):
                store = None
                account = None
                amount = 0
                sum = 0
                for item in items:
                    if item.get('Account.StoreOrAccount') == 'ACCOUNT':
                        account = item.get('Account.Id')
                        amount += item.get('Amount')
                        sum += item.get('Sum.Incoming')
                        if not main_item:
                            main_item = item
                    else:
                        store = item.get('Account.Id')

                document_items.append(ReturningInvoiceItem(
                    store=store,
                    account=account,
                    num=str(index + 1),
                    product=product,
                    amount=amount,
                    sum=sum
                ))

            result.append(ReturningInvoice(
                items=document_items,
                number=main_item['Document'],
                id=f'{main_item['Document']}_{main_item['DateTime.Typed']}',
                date=datetime.fromisoformat(main_item['DateTime.Typed']),
                account=main_item['Account.Id'],
                counterparty=main_item['Counteragent.Id'],
                conception_code=main_item['Conception.Code']
            ))

        return result




import json
from collections import defaultdict
from datetime import datetime

from iikoserver_api.schemas.outgoing_service import OutgoingServiceItem, OutgoingService


class OutgoingServiceBuilder:
    def __init__(self, items):
        self.items = items

    def process(self):
        documents = defaultdict(dict)
        for item in self.items:
            document_id = f'{item.get('Document')}_{item.get('DateTime.Typed')}_4_{item.get('Session.GroupId', '0')}'
            if not documents[document_id].get(item.get('Product.Id')):
                documents[document_id][item.get('Product.Id')] = []
            documents[document_id][item.get('Product.Id')].append(item)

        result = []
        for document, product in documents.items():
            document_items = []
            main_document = None
            for num, (product_id, items) in enumerate(product.items()):
                needed_items = [item for item in items if item.get('Sum.Outgoing')]
                if needed_items:
                    if not main_document:
                        main_document = needed_items[0]
                else:
                    continue

                data = {
                    'num': str(num + 1),
                    'product': product_id,
                    'account': needed_items[0]['Account.Id'],
                    'amount': 0,
                    'sum': 0
                }
                for item in items:
                    data['amount'] += item.get('Amount.Out')
                    data['sum'] += item.get('Sum.Incoming')

                document_items.append(OutgoingServiceItem(
                    **data
                ))

            if document_items and main_document:
                result.append(OutgoingService(
                    id=document,
                    date=datetime.fromisoformat(main_document.get('DateTime.Typed')),
                    num=main_document.get('Document'),
                    account=main_document.get('Account.Id'),
                    counterparty=main_document.get('Counteragent.Id'),
                    items=document_items,
                    conception_code=main_document.get('Conception.Code'),
                    counterparty_type=main_document.get('Account.CounteragentType')
                ))

        return result




from collections import defaultdict

from iikoserver_api.schemas.inventory import Inventory, InventoryItem, InventoryItemEnum


class InventoryDocumentBuilder:
    def __init__(self, items):
        self.items = items

    def process(self):
        documents = defaultdict(list)
        for item in self.items:
            if item.get('documentId'):
                documents[item['documentId']].append(item)

        result = []
        for document, items in documents.items():
            if len(items):
                main_document = items[0]
            else:
                continue
            document_items = []
            incoming_sum = 0
            outgoing_sum = 0
            incoming_account = None
            outgoing_account = None
            for item in items:
                sum = round(float(item['sum']), 2)
                if sum >= 0:
                    incoming_sum += sum
                    if not incoming_account:
                        incoming_account = item.get('secondaryAccount')
                else:
                    outgoing_sum += abs(sum)
                    if not outgoing_account:
                        outgoing_account = item.get('secondaryAccount')

            document_items.append(InventoryItem(
                type=InventoryItemEnum.INCOMING,
                sum=incoming_sum,
                account=incoming_account
            ))
            document_items.append(InventoryItem(
                type=InventoryItemEnum.OUTGOING,
                sum=outgoing_sum,
                account=outgoing_account
            ))

            result.append(Inventory(
                id=main_document['documentId'],
                store=main_document.get('primaryStore'),
                num=main_document.get('documentNum'),
                date=main_document.get('date'),
                incoming_sum=incoming_sum,
                outgoing_sum=outgoing_sum,
                sum=round(incoming_sum-outgoing_sum, 2),
                items=document_items
            ))

        return result



